# coding:utf-8
import numpy as np
import pandas as pd
from pandas import read_csv
from sklearn import preprocessing
from sklearn.model_selection import KFold
from sklearn.tree import DecisionTreeClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.neighbors import KNeighborsClassifier
from pandas import read_excel,read_csv
import matplotlib.pyplot as plt
from sklearn import preprocessing
from sklearn.model_selection import train_test_split
from sklearn.model_selection import cross_val_score
from sklearn.metrics import accuracy_score,roc_curve,f1_score,auc,matthews_corrcoef
from sklearn.grid_search import GridSearchCV

def sensitivity_specificity(y_true,y_pred):
    from sklearn.metrics import confusion_matrix
    tn, fp, fn, tp = confusion_matrix(y_true,y_pred).ravel()
    se = tp/(tp+fn)
    sp = tn/(tn+fp)
    return se, sp

def acc_auc_f1_se_sp_mcc(y_true,y_predict):
    acc = accuracy_score(y_true, y_predict)
    print("The ACC-Score: %.4f" % acc)

    fpr, tpr, _ = roc_curve(y_true,y_predict, pos_label=1)
    auc_score = auc(fpr, tpr)
    print('The AUC-Score: %.4f' % auc_score)

    f1 = f1_score(y_true,y_predict, average='binary')
    print('The F1-Score: %.4f' % f1)

    se, sp = sensitivity_specificity(y_true, y_predict)
    print('The Se-Score: %.4f' % se)
    print('The Sp-Score: %.4f' % sp)
    mcc = matthews_corrcoef(y_true, y_predict)
    print('The Mcc-Score: %.4f' % mcc)
    print('\n')
    return acc,auc_score,f1,se,sp,mcc

pos = read_csv('molecular.csv')
pos['target'] = 1
neg = read_csv('molecular.csv')
neg['target'] = 0

data = pd.DataFrame()
data = data.append(pos)
data = data.append(neg)
print(data.shape,pos.shape,neg.shape)
data.dropna(0, inplace = True)
data.drop('Name', axis = 1, inplace = True)

data = np.array(data)
np.random.shuffle(data)

X = data[:,:-1]
y = data[:,-1].astype('int')
std = preprocessing.StandardScaler()
X = std.fit_transform(X)
mstd = preprocessing.MinMaxScaler()
X = mstd.fit_transform(X)

X_train, X_test, y_train, y_test = train_test_split(X, y,test_size=0.2,shuffle = True)

#______________LR__________________________________
print('---------------------LR--------------------------------------')
from sklearn.linear_model import LogisticRegression
lr = LogisticRegression()
lr.fit(X_train, y_train)
y_lr_p = lr.predict(X_train)
y_lr_p_test = lr.predict(X_test)

acc_auc_f1_se_sp_mcc(y_train,y_lr_p)
acc_auc_f1_se_sp_mcc(y_test,y_lr_p_test) 

#______________SVC__________________________________________
print('---------------------SVC--------------------------------------')
from sklearn.svm import SVC
svc = SVC(kernel = 'linear')
svc.fit(X_train, y_train)
y_svc_p = svc.predict(X_train)
y_svc_p_test = svc.predict(X_test)

acc_auc_f1_se_sp_mcc(y_train,y_svc_p)
acc_auc_f1_se_sp_mcc(y_test,y_svc_p_test)

#______________GBDT__________________________________
print('---------------------GBDT--------------------------------------')
from sklearn import ensemble
gbdt = ensemble.GradientBoostingClassifier()

gbdt.fit(X_train, y_train)
y_gbdt_p = gbdt.predict(X_train)
y_gbdt_p_test = gbdt.predict(X_test)

acc_auc_f1_se_sp_mcc(y_train,y_gbdt_p)
acc_auc_f1_se_sp_mcc(y_test,y_gbdt_p_test)

#______________ADB__________________________________
print('---------------------ADB--------------------------------------')
adb = ensemble.AdaBoostClassifier()

adb.fit(X_train, y_train)
y_adb_p = adb.predict(X_train)
y_adb_p_test = adb.predict(X_test)

acc_auc_f1_se_sp_mcc(y_train,y_adb_p)
acc_auc_f1_se_sp_mcc(y_test,y_adb_p_test)

#______________XGB__________________________________
print('---------------------XGB--------------------------------------')
from xgboost.sklearn import XGBClassifier
xgbc = XGBClassifier()
xgbc.fit(X_train,y_train)
y_xgbc_p = xgbc.predict(X_train)
y_xgbc_p_test = xgbc.predict(X_test)
acc_auc_f1_se_sp_mcc(y_train,y_xgbc_p)
acc_auc_f1_se_sp_mcc(y_test,y_xgbc_p_test)

#----------- Random Forest---------------------------------------------
print('--------------Random Forest-------------------------------------')
from sklearn.ensemble import RandomForestClassifier
random_forest = RandomForestClassifier(n_estimators = 100 )
random_forest.fit(X_train, y_train)
y_rf_p = random_forest.predict(X_train)
y_rf_p_test = random_forest.predict(X_test)


acc_auc_f1_se_sp_mcc(y_train,y_rf_p)
acc_auc_f1_se_sp_mcc(y_test,y_rf_p_test)


#_________________ExtraTrees_____________________________________________________
print('--------------ExtraTrees-------------------------------------')
from sklearn.ensemble import ExtraTreesClassifier
ExtraT = ExtraTreesClassifier()
ExtraT.fit(X_train,y_train)
y_ExtraT_p = ExtraT.predict(X_train)
y_ExtraT_p_test = ExtraT.predict(X_test)

acc_auc_f1_se_sp_mcc(y_train,y_ExtraT_p)
acc_auc_f1_se_sp_mcc(y_test,y_ExtraT_p_test)


#______________LGB__________________________________
print('--------------LGB-------------------------------------')
import lightgbm as lgb
Lgb = lgb.LGBMClassifier()
Lgb.fit(X_train,y_train)
y_Lgb_p = Lgb.predict(X_train)
y_Lgb_p_test = Lgb.predict(X_test)

acc_auc_f1_se_sp_mcc(y_train,y_Lgb_p)
acc_auc_f1_se_sp_mcc(y_test,y_Lgb_p_test)

#______________CatB__________________________________
print('--------------CatB-------------------------------------')
from catboost import CatBoostClassifier,Pool
catb = CatBoostClassifier()

catb.fit(X_train,y_train)
y_catb_p = catb.predict(X_train)
y_catb_p_test = catb.predict(X_test)

acc_auc_f1_se_sp_mcc(y_train,y_catb_p)
acc_auc_f1_se_sp_mcc(y_test,y_catb_p_test)

