#coding:utf-8
###############################################################################
###########################  12 FPs 筛选 #######################################
###############################################################################
import numpy as np
import pandas as pd
from pandas import read_csv

from sklearn import preprocessing
from sklearn.model_selection import train_test_split
from sklearn.cross_validation import cross_val_score
from sklearn.model_selection import StratifiedKFold
from sklearn.metrics import accuracy_score,roc_curve,f1_score,matthews_corrcoef,classification_report,auc
from sklearn.grid_search import GridSearchCV

import pickle

from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import GradientBoostingClassifier
from xgboost.sklearn import XGBClassifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.ensemble import ExtraTreesClassifier
from lightgbm import LGBMClassifier
from catboost import CatBoostClassifier,Pool

def sensitivity_specificity(y_true,y_pred):
    from sklearn.metrics import confusion_matrix
    tn, fp, fn, tp = confusion_matrix(y_true,y_pred).ravel()
    se = tp/(tp+fn)
    sp = tn/(tn+fp)
    return se, sp

def acc_auc_f1_se_sp_mcc(y_true,y_predict):
    acc = accuracy_score(y_true, y_predict)
    print("The ACC-Score: %.4f" % acc)

    fpr, tpr, _ = roc_curve(y_true,y_predict, pos_label=1)
    auc_score = auc(fpr, tpr)
    print('The AUC-Score: %.4f' % auc_score)

    f1 = f1_score(y_true,y_predict, average='binary')
    print('The F1-Score: %.4f' % f1)

    se, sp = sensitivity_specificity(y_true, y_predict)
    print('The Se-Score: %.4f' % se)
    print('The Sp-Score: %.4f' % sp)
    mcc = matthews_corrcoef(y_true, y_predict)
    print('The Mcc-Score: %.4f' % mcc)
    print('\n')
    return acc,auc_score,f1,se,sp,mcc

def K_fold_definate_FPs(model,dataX,dataY,k=5):
    print('--------------------------5-K-fold-------------------------------')
    i = 1
    ACC, AUC, F1, SE, SP, MCC, Y_proba,Y_val = [], [], [], [], [], [],[],[]
    skf = StratifiedKFold(n_splits=k)
    for train_index, val_index in skf.split(dataX,dataY):
        X_train, X_val = dataX[train_index], dataX[val_index]
        y_train, y_val = dataY[train_index], dataY[val_index]
        print('-------------------The',i,'-fold--------------------------------------')
        model.fit(X_train, y_train)

        y_val_pred = model.predict(X_val)
        y_prob = model.predict_proba(X_val)[:,-1]

        Y_proba.append(y_prob)
        Y_val.append(y_val)

        acc = accuracy_score(y_val, y_val_pred)
        ACC.append(acc)
        fpr, tpr, _ = roc_curve(y_val, y_val_pred, pos_label=1)
        auc_score = auc(fpr, tpr)
        AUC.append(auc_score)
        f1 = f1_score(y_val, y_val_pred, average='binary')
        F1.append(f1)
        se, sp = sensitivity_specificity(y_val, y_val_pred)
        SE.append(se)
        SP.append(sp)
        mcc = matthews_corrcoef(y_val, y_val_pred)
        MCC.append(mcc)
        i = i+1

    print('ACC:%.4f' %  np.array(ACC).mean(),
          'AUC:%.4f' %  np.array(AUC).mean(),
          'F1:%.4f' %  np.array(F1).mean(),
          'SE:%.4f' %  np.array(SE).mean(),
          'SP:%.4f' %  np.array(SP).mean(),
          'MCC:%.4f' %  np.array(MCC).mean())
    Y_pred_P = pd.DataFrame(Y_proba)
    Y_val = pd.DataFrame(Y_val)
    return (Y_pred_P,Y_val,np.array(ACC),np.array(AUC),np.array(F1),np.array(SE),np.array(SP),np.array(MCC))

def main():
    ##################################################################################
    ##################################################################################
    pos = read_csv('molecular.csv')
    neg = read_csv('molecular.csv')
    ##################################################################################
    pos['target'] = 1
    neg['target'] = 0

    data = pd.DataFrame()
    data = data.append(pos)
    data = data.append(neg)

    print(data.shape, pos.shape, neg.shape)
    data.dropna(0, inplace = True)
    data.drop('Name', axis = 1, inplace = True)

    data = np.array(data)
    np.random.shuffle(data)

    X = data[:, :-1]
    y = data[:, -1].astype('int')
    std = preprocessing.StandardScaler()
    X = std.fit_transform(X)
    mstd = preprocessing.MinMaxScaler()
    X = mstd.fit_transform(X)

    gbdt_FPs = GradientBoostingClassifier(learning_rate = 0.1,n_estimators = 100,max_features='sqrt',max_depth = 10,min_samples_split = 10,random_state = 50)
    xgbc_FPs = XGBClassifier(gamma = 0.001,learning_rate = 0.2,n_estimators=90,max_depth=8)
    rf_FPs = RandomForestClassifier(n_estimators = 90,max_features='sqrt')
    Lgb_FPs = LGBMClassifier(boosting_type='gbdt',learning_rate = 0.08,max_depth=11,num_leaves=20)
    catb_FPs = CatBoostClassifier(iterations=900, depth=12, learning_rate=0.1, l2_leaf_reg=5)

    ############################  Val #################################################
    # ####---------voting models--------------------------------------------------------
    print("---------------------ValData-voting------------------------------------")
    from sklearn.ensemble import VotingClassifier
    voting_FPs = VotingClassifier(estimators = [
            ('RF_clf',rf_FPs),
            ('GBDT_clf',gbdt_FPs),
            ('XGB_clf',xgbc_FPs),
            ('lgb_clf',Lgb_FPs),
            ('catb_clf',catb_FPs)],
            voting = 'soft',weights=[2,1,1,1,2])

    Prob1,y_val,_,_,_,_,_,_ = K_fold_definate_FPs(voting_FPs,X,y,k=5)

if __name__ == '__main__':
    main()