#coding:utf-8
import numpy as np
import pandas as pd
from pandas import read_csv

from sklearn import preprocessing
from sklearn.model_selection import train_test_split
from sklearn.cross_validation import cross_val_score
from sklearn.model_selection import StratifiedKFold
from sklearn.metrics import accuracy_score,roc_curve,f1_score,matthews_corrcoef,classification_report,auc
from sklearn.grid_search import GridSearchCV

import pickle

from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import GradientBoostingClassifier
from xgboost.sklearn import XGBClassifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.ensemble import ExtraTreesClassifier
from lightgbm import LGBMClassifier
from catboost import CatBoostClassifier,Pool

def sensitivity_specificity(y_true,y_pred):
    from sklearn.metrics import confusion_matrix
    tn, fp, fn, tp = confusion_matrix(y_true,y_pred).ravel()
    se = tp/(tp+fn)
    sp = tn/(tn+fp)
    return se, sp

def acc_auc_f1_se_sp_mcc(y_true,y_predict):
    acc = accuracy_score(y_true, y_predict)
    print("The ACC-Score: %.4f" % acc)

    fpr, tpr, _ = roc_curve(y_true,y_predict, pos_label=1)
    auc_score = auc(fpr, tpr)
    print('The AUC-Score: %.4f' % auc_score)

    f1 = f1_score(y_true,y_predict, average='binary')
    print('The F1-Score: %.4f' % f1)

    se, sp = sensitivity_specificity(y_true, y_predict)
    print('The Se-Score: %.4f' % se)
    print('The Sp-Score: %.4f' % sp)
    mcc = matthews_corrcoef(y_true, y_predict)
    print('The Mcc-Score: %.4f' % mcc)
    print('\n')
    return acc,auc_score,f1,se,sp,mcc

def Save_Model(model,modelname): 
    import os
    import datetime
    nowTime = datetime.datetime.now().strftime('%Y-%m-%d %H-%M-%S')
    path = "./model"
    pathfile = path + "/" + modelname + ".pickle"
    new_file = path + "/" + modelname + " " + nowTime + ".pickle"

    if not (os.path.exists(path) and os.path.isdir(path)):
        os.makedirs(path)
    if os.path.exists(pathfile) and os.path.isfile(pathfile):
        os.rename(pathfile,new_file)
    with open(pathfile, 'wb') as fw: 
        pickle.dump(model, fw)

def Load_Model(model_file): 
    loaded_model = pickle.load(open(("./model/"+model_file), "rb"))
    return loaded_model

def K_fold_definate_FPs(model,dataX,dataY,k=5):
    print('--------------------------5-K-fold-------------------------------')
    i = 1
    Y_proba,Y_val = [], []
    skf = StratifiedKFold(n_splits=k)
    for train_index, val_index in skf.split(dataX,dataY):
        X_train, X_val = dataX[train_index], dataX[val_index]
        y_train, y_val = dataY[train_index], dataY[val_index]
        print('-------------------The',i,'-fold--------------------------------------')
        model.fit(X_train, y_train)
        ###############################################################################
        Save_Model(model, "model_FPs"+str(i))
        model = Load_Model("model_FPs"+str(i)+".pickle")
        y_val_pred = model.predict(X_val)
        y_prob = model.predict_proba(X_val)[:,-1]

        Y_proba.append(y_prob)
        Y_val.append(y_val)

    Y_pred_P = pd.DataFrame(Y_proba)
    Y_val = pd.DataFrame(Y_val)
    return (Y_pred_P,Y_val)

def K_fold_definate_7Mols(model,dataX,dataY,k=5):
    print('--------------------------5-K-fold-------------------------------')
    i = 1
    Y_proba, Y_val = [], []
    skf = StratifiedKFold(n_splits=k)
    for train_index, val_index in skf.split(dataX,dataY):
        X_train, X_val = dataX[train_index], dataX[val_index]
        y_train, y_val = dataY[train_index], dataY[val_index]
        print('-------------------The',i,'-fold--------------------------------------')
        model.fit(X_train, y_train)
        #############################################################################
        Save_Model(model, "model_7Mols"+str(i))
        model = Load_Model("model_7Mols"+str(i)+".pickle")
        y_val_pred = model.predict(X_val)
        y_prob = model.predict_proba(X_val)[:,-1]

        Y_proba.append(y_prob)
        Y_val.append(y_val)

    Y_pred_P = pd.DataFrame(Y_proba)
    Y_val = pd.DataFrame(Y_val)
    return (Y_pred_P,Y_val)

def weights_compose(y_test_p1,y_test_p2, w1=0.5, w2=0.5):
    pred = []
    target = 0
    for i in range(len(y_test_p1)):
        proba = w1 * y_test_p1[i] + w2 * y_test_p2[i]
        if proba >= 0.53:
            target = 1
        else:
            target = 0
        pred.append(target)
    pred = pd.DataFrame(pred)
    return pred

def main():
    data_train = read_csv('data_train.csv')
    data_test = read_csv('data_test.csv')
    data_test.fillna(0,inplace = True)
    data_train = np.array(data_train)
    data_test = np.array(data_test)

    X = data_train[:,:-1]
    y = data_train[:,-1].astype('int')

    test_X = data_test[:,:-1]
    test_y = data_test[:,-1].astype('int')

    std = preprocessing.StandardScaler()
    X = std.fit_transform(X)
    test_X = std.transform(test_X)
    mstd = preprocessing.MinMaxScaler()
    X = mstd.fit_transform(X)
    test_X = mstd.transform(test_X)
    ################################################################################
    ################################################################################
    ################################################################################
    X_FPs = X[:,:780]
    X_7Mols = X[:,780:]
    ################################################################################

    print(X_FPs.shape,X_7Mols.shape)
    ################################################################################
    ################################################################################
    ################################################################################
    test_X_FPs = test_X[:,:780]
    test_X_7Mols = test_X[:,780:]
    ################################################################################

    gbdt_FPs = GradientBoostingClassifier(learning_rate = 0.1,n_estimators = 100,max_features='sqrt',max_depth = 10,min_samples_split = 10,random_state = 50)
    gbdt_7Mols = GradientBoostingClassifier(learning_rate = 0.1,n_estimators=50,max_features='sqrt',max_depth = 7,min_samples_split =7,random_state = 50)
    #######______________XGB__________________________________
    print('---------------------XGB--------------------------------------')
    xgbc_FPs = XGBClassifier(gamma = 0.001,learning_rate = 0.2,n_estimators=90,max_depth=8)
    xgbc_7Mols = XGBClassifier(gamma = 0.001,learning_rate = 0.1,n_estimators=80,max_depth=8)
    #----------- Random Forest---------------------------------------------
    print('-------------------Random Forest-------------------------------------')
    rf_FPs = RandomForestClassifier(n_estimators = 90,max_features='sqrt')
    rf_7Mols = RandomForestClassifier(n_estimators = 80,max_features='log2')
    # #______________LGB__________________________________
    print('-----------------------LGB-------------------------------------')
    Lgb_FPs = LGBMClassifier(boosting_type='gbdt',learning_rate = 0.08,max_depth=11,num_leaves=20)
    Lgb_7Mols = LGBMClassifier(boosting_type='gbdt',learning_rate = 0.06,max_depth=11,num_leaves=20)
    #______________CatB__________________________________
    print('-----------------------CatB-------------------------------------')
    catb_FPs = CatBoostClassifier(iterations=900, depth=12, learning_rate=0.1, l2_leaf_reg=5)
    catb_7Mols = CatBoostClassifier(iterations=600, depth=12, learning_rate=0.1, l2_leaf_reg=5)

    ############################  Val #################################################
    # ####---------voting models--------------------------------------------------------
    print("---------------------ValData-voting------------------------------------")
    from sklearn.ensemble import VotingClassifier
    voting_FPs = VotingClassifier(estimators = [
            ('RF_clf',rf_FPs),
            ('GBDT_clf',gbdt_FPs),
            ('XGB_clf',xgbc_FPs),
            ('lgb_clf',Lgb_FPs),
            ('catb_clf',catb_FPs)],
            voting = 'soft',weights=[2,1,1,1,2])

    Prob1,y_val = K_fold_definate_FPs(voting_FPs,X_FPs,y,k=5)

    voting_7Mols = VotingClassifier(estimators = [
            ('RF_clf',rf_7Mols),
            ('GBDT_clf',gbdt_7Mols),
            ('XGB_clf',xgbc_7Mols),
            ('lgb_clf',Lgb_7Mols),
            ('catb_clf',catb_7Mols)],
            voting = 'soft',weights=[2,1,1,1,2])

    Prob2,y_val = K_fold_definate_7Mols(voting_7Mols,X_7Mols,y,k=5)

    ACC_,AUC_,F1_,SE_,SP_,MCC_ = [],[],[],[],[],[]
    ################################################################################
    ################################################################################
    w_FPs = 0.7
    w_7Mols = 0.3
    ################################################################################
    for i in range(5):
        weight_pred = weights_compose(Prob1.loc[i,:],Prob2.loc[i,:], w1=w_FPs, w2=w_7Mols)
        print("--------------------voting_FPs-voting_7Mols-weights-",i+1,"--------------------------------")
        acc_,auc_,f1_,se_,sp_,mcc_ = acc_auc_f1_se_sp_mcc(y_val.loc[i,:],weight_pred)
        ACC_.append(acc_)
        AUC_.append(auc_)
        F1_.append(f1_)
        SE_.append(se_)
        SP_.append(sp_)
        MCC_.append(mcc_)

    print('------------------ValData--Last--voting_FPs-voting_7Mols-weights-----------------------------')
    print('ACC: %.4f' % np.array(ACC_).mean(),'\n',
        'AUC: %.4f' % np.array(AUC_).mean(),'\n',
        'F1:  %.4f' % np.array(F1_).mean(),'\n',
        'SE:  %.4f' % np.array(SE_).mean(),'\n',
        'SP:  %.4f' % np.array(SP_).mean(),'\n',
        'MCC: %.4f' % np.array(MCC_).mean(),'\n')

    ##################################################################################
    ########################  TEST   #################################################
    # ####---------voting models--------------------------------------------------------
    print("---------------------TestData-voting------------------------------------")
    ACC, AUC, F1, SE, SP, MCC, Y_proba1 = [], [], [], [], [], [], []
    for i in range(5):
        model = Load_Model("model_FPs"+str(i+1)+".pickle")
        y_test_pred = model.predict(test_X_FPs)
        y_prob = model.predict_proba(test_X_FPs)[:,-1]
        Y_proba1.append(y_prob)

    Prob1= pd.DataFrame(Y_proba1)
    test_y = pd.DataFrame(test_y)

    ACC, AUC, F1, SE, SP, MCC, Y_proba2 = [], [], [], [], [], [], []
    for i in range(5):
        model = Load_Model("model_7Mols" + str(i+1) + ".pickle")
        y_test_pred = model.predict(test_X_7Mols)
        y_prob = model.predict_proba(test_X_7Mols)[:, -1]
        Y_proba2.append(y_prob)

    Prob2 = pd.DataFrame(Y_proba2)
    test_y = pd.DataFrame(test_y)

    ACC_,AUC_,F1_,SE_,SP_,MCC_ = [],[],[],[],[],[]

    for i in range(5):
        weight_pred = weights_compose(Prob1.loc[i,:],Prob2.loc[i,:], w1=0.7, w2=0.3)
        print("--------------------voting_FPs-voting_7Mols-weights-",i+1,"--------------------------------")
        acc_,auc_,f1_,se_,sp_,mcc_ = acc_auc_f1_se_sp_mcc(test_y,weight_pred)
        ACC_.append(acc_)
        AUC_.append(auc_)
        F1_.append(f1_)
        SE_.append(se_)
        SP_.append(sp_)
        MCC_.append(mcc_)

    print('------------------ValData--Last--voting_FPs-voting_7Mols-weights-----------------------------')
    print('ACC: %.4f' % np.array(ACC_).mean(),'\n',
        'AUC: %.4f' % np.array(AUC_).mean(),'\n',
        'F1:  %.4f' % np.array(F1_).mean(),'\n',
        'SE:  %.4f' % np.array(SE_).mean(),'\n',
        'SP:  %.4f' % np.array(SP_).mean(),'\n',
        'MCC: %.4f' % np.array(MCC_).mean(),'\n')

if __name__ == '__main__':
    main()