# coding:utf-8
import numpy as np
import pandas as pd
from pandas import read_csv
from sklearn import preprocessing
from sklearn.model_selection import KFold
from sklearn.tree import DecisionTreeClassifier
from sklearn.linear_model import LogisticRegression
from sklearn.neighbors import KNeighborsClassifier
from pandas import read_excel,read_csv
import matplotlib.pyplot as plt
from sklearn import preprocessing
from sklearn.model_selection import train_test_split
from sklearn.model_selection import cross_val_score
from sklearn.metrics import accuracy_score,roc_curve,f1_score,auc,matthews_corrcoef
from sklearn.grid_search import GridSearchCV

def sensitivity_specificity(y_true,y_pred):
    from sklearn.metrics import confusion_matrix
    tn, fp, fn, tp = confusion_matrix(y_true,y_pred).ravel()
    se = tp/(tp+fn)
    sp = tn/(tn+fp)
    return se, sp

def acc_auc_f1_se_sp_mcc(y_true,y_predict):
    acc = accuracy_score(y_true, y_predict)
    print("The ACC-Score: %.4f" % acc)

    fpr, tpr, _ = roc_curve(y_true,y_predict, pos_label=1)
    auc_score = auc(fpr, tpr)
    print('The AUC-Score: %.4f' % auc_score)

    f1 = f1_score(y_true,y_predict, average='binary')
    print('The F1-Score: %.4f' % f1)

    se, sp = sensitivity_specificity(y_true, y_predict)
    print('The Se-Score: %.4f' % se)
    print('The Sp-Score: %.4f' % sp)
    mcc = matthews_corrcoef(y_true, y_predict)
    print('The Mcc-Score: %.4f' % mcc)
    print('\n')
    return acc,auc_score,f1,se,sp,mcc


data_train = read_csv('data_train.csv')
data_test = read_csv('data_test.csv')
data_train = np.array(data_train)
data_test = np.array(data_test)
np.random.shuffle(data_train)

X = data_train[:,:-1]
y = data_train[:,-1].astype('int')

test_X = data_test[:,:-1]
test_y = data_test[:,-1].astype('int')

std = preprocessing.StandardScaler()
X = std.fit_transform(X)
test_X = std.transform(test_X)
mstd = preprocessing.MinMaxScaler()
X = mstd.fit_transform(X)
test_X = mstd.transform(test_X)



#______________LR__________________________________
print('---------------------LR--------------------------------------')
from sklearn.linear_model import LogisticRegression
lr = LogisticRegression()
lr.fit(X, y)
y_lr_p = lr.predict(X)
y_lr_p_test = lr.predict(test_X)

acc_auc_f1_se_sp_mcc(y,y_lr_p)
acc_auc_f1_se_sp_mcc(test_y,y_lr_p_test) 

#______________SVC__________________________________________
print('---------------------SVC--------------------------------------')
from sklearn.svm import SVC
svc = SVC(kernel = 'linear')
svc.fit(X, y)
y_svc_p = svc.predict(X)
y_svc_p_test = svc.predict(test_X)

acc_auc_f1_se_sp_mcc(y,y_svc_p)
acc_auc_f1_se_sp_mcc(test_y,y_svc_p_test) 

#______________GBDT__________________________________
print('---------------------GBDT--------------------------------------')
from sklearn import ensemble
gbdt = ensemble.GradientBoostingClassifier()

gbdt.fit(X, y)
y_gbdt_p = gbdt.predict(X)
y_gbdt_p_test = gbdt.predict(test_X)

acc_auc_f1_se_sp_mcc(y,y_gbdt_p)
acc_auc_f1_se_sp_mcc(test_y,y_gbdt_p_test)

#______________ADB__________________________________
print('---------------------ADB--------------------------------------')
adb = ensemble.AdaBoostClassifier()

adb.fit(X, y)
y_adb_p = adb.predict(X)
y_adb_p_test = adb.predict(test_X)

acc_auc_f1_se_sp_mcc(y,y_adb_p)
acc_auc_f1_se_sp_mcc(test_y,y_adb_p_test)

#______________XGB__________________________________
print('---------------------XGB--------------------------------------')
from xgboost.sklearn import XGBClassifier
#cv_params = {'learning_rate': [0.01, 0.03,0.04,0.05, 0.06,0.07, 0.1]}
xgbc = XGBClassifier()
xgbc.fit(X,y)
y_xgbc_p = xgbc.predict(X)
y_xgbc_p_test = xgbc.predict(test_X)
acc_auc_f1_se_sp_mcc(y,y_xgbc_p)
acc_auc_f1_se_sp_mcc(test_y,y_xgbc_p_test)

#----------- Random Forest---------------------------------------------
print('--------------Random Forest-------------------------------------')
from sklearn.ensemble import RandomForestClassifier
random_forest = RandomForestClassifier(n_estimators = 100 )
random_forest.fit(X, y)
y_rf_p = random_forest.predict(X)
y_rf_p_test = random_forest.predict(test_X)
# y_rf_p_val = random_forest.predict(val_X_temp)
# y_rf_test_pro = random_forest.predict_proba(X_test)[:,-1]

acc_auc_f1_se_sp_mcc(y,y_rf_p)
acc_auc_f1_se_sp_mcc(test_y,y_rf_p_test)


#_________________ExtraTrees_____________________________________________________
print('--------------ExtraTrees-------------------------------------')
from sklearn.ensemble import ExtraTreesClassifier
ExtraT = ExtraTreesClassifier()
ExtraT.fit(X,y)
y_ExtraT_p = ExtraT.predict(X)
y_ExtraT_p_test = ExtraT.predict(test_X)

acc_auc_f1_se_sp_mcc(y,y_ExtraT_p)
acc_auc_f1_se_sp_mcc(test_y,y_ExtraT_p_test)


#______________LGB__________________________________
print('--------------LGB-------------------------------------')
import lightgbm as lgb
#cv_params = {'learning_rate': [0.01, 0.03,0.04,0.05, 0.06,0.07, 0.08,0.1]}
Lgb = lgb.LGBMClassifier()
Lgb.fit(X,y)
y_Lgb_p = Lgb.predict(X)
y_Lgb_p_test = Lgb.predict(test_X)

acc_auc_f1_se_sp_mcc(y,y_Lgb_p)
acc_auc_f1_se_sp_mcc(test_y,y_Lgb_p_test)

#______________CatB__________________________________
print('--------------CatB-------------------------------------')
from catboost import CatBoostClassifier,Pool
catb = CatBoostClassifier()

catb.fit(X,y)
y_catb_p = catb.predict(X)
y_catb_p_test = catb.predict(test_X)

acc_auc_f1_se_sp_mcc(y,y_catb_p)
acc_auc_f1_se_sp_mcc(test_y,y_catb_p_test)

